#!/bin/bash

# Purpose:
#    This script is made to create movie in mp4 format based on a 
#   list of PNG images.
#    The PNG images are first convert to jpg with the best qulity offered by imagemagick
#   and then turn into a movie using ffmpeg
#
# Input:
#    none
#
# Output:
#    a mp4 files using x264 codec
#
# Restrictions:
#    the maximum number of PNG images is 2000
#
# Comments:
#    The script create a temporary directory in the output directory 
#   to store the JPG images which is erased after the creation of the movie
#    If you can't read the movie created, try using VLC (videolan.org)
#    Don't forget to make the this script executable before using it (chmod u+x convert_PNG_to_VIDEO.sh)    
#
# Requirement:
#    this script require:
#		- imagemagick (http://www.imagemagick.org/)
#       - ffmpeg (ffmpeg)
#
# History:
#    201104 MAD/LPC2E Creation
#    201109 MAD/LPC2E Add settings part
#                     Add option to set the resolution of the movie
#                     Add copy of each images to have at least 10 FPS
#                     
# Author: M.-A. Drouin

#----------------------------------------------------------------------
# Settings
#----------------------------------------------------------------------

# Directory were the PNG images are stored
M_INDIR="/home/madrouin/test/IDL_Plot3D"
# Directory were the movie will be saved
M_OUTDIR="/home/madrouin/test/IDL_Plot3D"
# Name Pattern of the images which will converted to movie 
M_PATTERN="mimosa_3D_15"
# Name of the movie (without extension)
M_VNAME=test_mimosa_vortex
#   Number of frames per seconds
# each images is copied 5 times so for example
# if you want 3 of your images per seconds in your movie M_FPS=3images*5copies=15
# For compatibility with most of video player it is advise to never use a framerate below 10 FPS
M_FPS=10
# Resolution of the video
# It doesn't matter if your images have a bigger resolution ffmpeg will take care of the convertion
# horizontal resolution
M_XSIZE=1024
# vertical resolution
M_YSIZE=768

#----------------------------------------------------------------------
# Program 
#----------------------------------------------------------------------

# Initialization
M_VNAME=${M_VNAME}.mp4
SIZE=${M_XSIZE}x${M_YSIZE}
TMPDIR=${M_OUTDIR}/tmp
if [[ ! -d $TMPDIR ]]
then
	mkdir ${TMPDIR}
fi
cd ${TMPDIR}

#----------------------------------------------------------------------
# Convert PNG images to JPEG
# Necessary to create the video
#----------------------------------------------------------------------

PATTERN=${M_PATTERN}"*.png"
PNGFILES=`ls ${M_INDIR}/${PATTERN}`

declare -i count=1

# Convert PNG images into JPEG
for f in $PNGFILES
do
	if (( count < 10 ))
	then
		scount=$(echo 0000$count)
	elif (( count < 100 ))
	then
		scount=$(echo 000$count)
	elif (( count < 1000 ))
	then
		scount=$(echo 00$count)
	elif (( count < 10000 ))
	then
		scount=$(echo 0$count)
	fi

	out=${scount}.jpg
	convert -quality 100 ${f} ${out}
	(( count++ )) 
		
	#  we repeat each image 4 times so with 2 MIMOSA images per second 
	# we have at least 10 fps to prevent problems
	# of a to small framerate which make video incompatible 
	# with quicktime or VLC for example
	# the file won't be a lot bigger as there no change between these 5 images
	for j in 1 2 3 4
	do
		if (( count < 10 ))
		then
			scount=$(echo 0000$count)
		elif (( count < 100 ))
		then
			scount=$(echo 000$count)
		elif (( count < 1000 ))
		then
			scount=$(echo 00$count)
		elif (( count < 10000 ))
		then
			scount=$(echo 0$count)
		fi

		out2=${scount}.jpg
		ln -s $out $out2

		(( count++ ))
	
	done
done

# Creation of the movie
ffmpeg -r ${M_FPS} -threads 0 -i %05d.jpg -vcodec libx264 -vpre max -qmin 5 -qmax 5 -s ${SIZE} ${M_VNAME}

# Move the movie into M_OUTDIR
mv ${M_VNAME} ${M_OUTDIR}

# Delete the tmp directory
cd ${M_OUTDIR}
\rm -rf $TMPDIR 

exit 0