;+
; :Description:
;    This procedure is made to read binary FORTRAN UNFORMATTED files
;    produce by MIMOSA model.
;    - potential vorticity files
;    - temperature files
;    - wind horizontal conponents files
;    
; :Categories:
;    MIMOSA tools
;
; :Params:
;    inFile : In, Required, Type=string
;            The Path+Name of the file
;
; :Keywords:
;    HEADER : Out, Optional, Type=lonarr
;            This variable contains the header of the read file
;            
;    DATA : Out, Optional, Type=fltarr
;          This variable contains the PV, temperature or wind field read in the file
;              
;    DIMX : Out, Optional, Type=long
;          The number of grid points along longitude
;          
;    DIMY : Out, Optional, Type=Long
;          The number of grid points along latitude
;          
;  :Examples:
;    inFile = '/home/madrouin/MIMOSA/RUN00/pvn11040312.0850'
;    MIMOSA_READ_BIN, inFile, DATA=pv, HEADER=header, DIMX=nLon, DIMY=nLat
;    
;  :File_comments:
;    none
;    
;  :History:
;    200904 MAD/LPC2E creation
;    201006 MAD/LPC2E add keyword HEADER
;    201109 MAD/LPC2E add comments
;
; :Author: madrouin
;-
PRO MIMOSA_READ_BIN, inFile $
    , HEADER = header $
    , DATA   = data $
    , DIMX   = Dimx $
    , DIMY   = Dimy
  
  ; Initialization   
  dummy = LONARR(1)
  header  = LONARR(30)
  
  ; Open the file
  OPENR, lun, inFile, /GET_LUN
  
  ; Read header
  READU, lun, dummy, header
  
  ; Get grid dimension from file header
  Dimx = header[19]
  Dimy = header[20]
  
  ; Read data
  data = FLTARR(Dimx, Dimy)
  READU, lun, data
  
  CLOSE, lun
  FREE_LUN, lun
  
END